<?php
namespace Controller;

use Util\InstanceProvider;
use Util\Messages;
use Util\Utilities;
use Zeedhi\Framework\Controller\Simple;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\File;
use Zeedhi\DevOps\Util\Utilities as DevopsUtilities;
use Zeedhi\Framework\Routing\Exception;

class Project extends Simple {
    /**
     * @var \Service\Project
     */
    public function __construct() {

        \Util\InstanceProvider::getLog()->setLog('instancinando project controller',false,2);
        \Util\InstanceProvider::getLog()->setLog(' + total de instancias',false,3);
    }

    public function findPublicProjectByUser(Request\Filter $request,Response $response){
        $dataSet = InstanceProvider::getService()->getProject()->findPublicProjectByUser();
        $response->addDataset(new DataSet('projects', $dataSet));
    }

    public function recalculateProject(Request\Filter $request,Response $response){
        try{
            $filters = $request->getFilterCriteria()->getConditions();
            $projectId = $filters[0]['value'];
            InstanceProvider::getService()->getProject()->recalculateProject($projectId);
        }catch (\Exception $e){
            $response->addMessage(new Response\Message('Erro inesperado :'.$e->getMessage()));
        }
    }

    public function recalculateTasks(Request\Filter $request,Response $response){
        try{
            $filters = $request->getFilterCriteria()->getConditions();
            $taskId = $filters[0]['value'];
            InstanceProvider::getService()->getProject()->recalculateTasks($taskId);
        }catch (\Exception $e){
            $response->addMessage(new Response\Message('Erro inesperado :'.$e->getMessage()));
        }
    }

    public function findAllProjectParent(Request\Filter $request,Response $response){
        $conditions = $request->getFilterCriteria()->getConditions();
        $projectId = empty($conditions[0]['value']) ? 0 : $conditions[0]['value'] ;
        $dataSet = InstanceProvider::getService()->getProject()->findAllProjectParent($projectId);
        $response->addDataset(new DataSet('projects', $dataSet));
    }

    public function checkProjectHasTemplateParentRecursive(Request\Filter $request, Response $response){
        $conditions = $request->getFilterCriteria()->getConditions();
        $projectId = DevOpsUtilities::getValueInFilterDataSourceByName($conditions, 'PROJECT_ID');
        $hasTemplateParent = InstanceProvider::getService()->getProject()->checkProjectHasTemplateParentRecursive($projectId);
        $productProject = InstanceProvider::getService()->getProject()->productProjectHasAllowReplicationLts($projectId);
        $response->addDataSet(new DataSet('checkProjectHasTemplateParentRecursive', array($hasTemplateParent)));
        $response->addDataSet(new DataSet('productProjectHasAllowReplicationLts', array($productProject)));
    }

    public function projectFinishPlanning(Request\DataSet $request,Response $response){
        try {
            $row = $request->getDataSet()->getRows()[0];
            $projectId = $row['ID'];
            InstanceProvider::getService()->getProject()->projectFinishPlanning($projectId);
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function createTasksByIssues(Request\DataSet $request, Response $response)
    {
        try {
            $param = $request->getDataSet()->getRows()[0];
            $issues = $param['ISSUES'];
            $allProjects = $param['ALL_PROJECTS'];
            $classificationid = empty($param['CLASSIFICATION']) ? array() : $param['CLASSIFICATION'];
            $productVersionDevId = empty($param['ALL_VERSION_DEV_ID']) ? array() : $param['ALL_VERSION_DEV_ID'];
            $projectProductIssueId = empty($param['PROJECT_PRODUCT_ISSUE_ID']) ? array() : $param['PROJECT_PRODUCT_ISSUE_ID'];
            $tasks = array();
            foreach ($allProjects as $projectId) {
                $tasks[] = array(
                    'projectId' => $projectId,
                    'resourceIds' => empty($param['RESOURCE_' . $projectId]) ? array() : $param['RESOURCE_' . $projectId],
                    'taskFeaturesId' => empty($param['TASK_FEATURES_ID_' . $projectId]) ? null : $param['TASK_FEATURES_ID_' . $projectId],
                    'timePlannedId' => empty($param['TIME_PLANNED_' . $projectId]) ? null : $param['TIME_PLANNED_' . $projectId],
                    'parentId' => empty($param['PARENT_' . $projectId]) ? null : $param['PARENT_' . $projectId],
                    'description' => empty($param['DESCRIPTION_' . $projectId]) ? null : $param['DESCRIPTION_' . $projectId],

                    );
            }
            $directoryBasicUpload = InstanceProvider::getEnvironment()->getDirectoryBasicUpload();
            $createdTaskIds = InstanceProvider::getService()->getProject()->createTasksByIssues($tasks, $issues, $directoryBasicUpload, true,$classificationid, $productVersionDevId, $projectProductIssueId);

            InstanceProvider::getService()->getTask()->applyOrderToIssuesCreateTasks($createdTaskIds, true);
            $response->addDataset(new DataSet('createTasksByIssues', $createdTaskIds));
            $response->addNotification(new Notification(\Util\Messages::SUC_SUCCESSFUL_CREATED_TASKS_BY_ISSUES, Notification::TYPE_SUCCESS));
        } catch (\Exception $e) {
            if ($e->getCode() == 2) {
                $e = new \Exception(Messages::ERR_CLASS_IN_HOMOLOGATION_ISSUE, $e->getCode());
            }
            throw $e;
        }
    }

    public function createTasksByTask(Request\DataSet $request, Response $response)
    {
        try {
            $param = $request->getDataSet()->getRows()[0];
            $primeTask = $param['TASKS'][0];
            $allProjects = $param['ALL_PROJECTS'];
            $tasks = array();
            foreach ($allProjects as $projectId) {
                $tasks[] = array(
                    'projectId' => $projectId,
                    'resourceIds' => empty($param['RESOURCE_' . $projectId]) ? array() : $param['RESOURCE_' . $projectId],
                    'parentId' => empty($param['PARENT_' . $projectId]) ? null : $param['PARENT_' . $projectId],
                );
            }
            $directoryBasicUpload = InstanceProvider::getEnvironment()->getDirectoryBasicUpload();
            InstanceProvider::getService()->getProject()->createTasksByTask($tasks,$primeTask, $directoryBasicUpload);
            $response->addNotification(new Notification(\Util\Messages::SUC_SUCCESSFUL_CREATED_TASKS_BY_ISSUES, Notification::TYPE_SUCCESS));
        } catch (\Exception $e) {
            if ($e->getCode() == 2) {
                $e = new \Exception(Messages::ERR_CLASS_IN_HOMOLOGATION_ISSUE, $e->getCode());
            }
            throw $e;
        }
    }

    public function calculationBasedStart(Request\DataSet $request,Response $response){
        try {
            $row = $request->getDataSet()->getRows()[0];
            list($task, $begin, $end, $duration, $parent, $exception, $exceptionDate,$group) =
                $this->getValuesCalculation($row);
            list($begin,$end, $exception, $exceptionDate) = InstanceProvider::getService()->getProject()->calculationBasedStart($task, $begin, $end, $duration,
                $parent, $exception, $exceptionDate,$group);
            $dataSet = array(array(
                'dateEnd' => $end->format(Utilities::FORMAT_DATE_DEFAULT_ONLY_DATE),
                'dateBegin' => $begin->format(Utilities::FORMAT_DATE_DEFAULT_ONLY_DATE),
                'exception' => $exception,
                'exceptionDate' => $exceptionDate,
            ));
            $response->addDataset(new DataSet('calculationDate', $dataSet));
        }catch (\Exception $e){
            $response->addMessage(new Response\Message('Erro inesperado '.$e->getMessage()));
            throw $e;
        }
    }
    public function calculationBasedEnd(Request\DataSet $request,Response $response){
        try {
            $row = $request->getDataSet()->getRows()[0];
            list($task, $begin, $end, $duration, $parent, $exception, $exceptionDate,$group) =
                $this->getValuesCalculation($row);
            list($begin,$end , $exception, $exceptionDate) = InstanceProvider::getService()->getProject()->calculationBasedEnd($task, $begin, $end, $duration,
                 $parent, $exception, $exceptionDate,$group);
            $dataSet = array(array(
                'dateBegin' => $begin->format(Utilities::FORMAT_DATE_DEFAULT_ONLY_DATE),
                'dateEnd' => $end->format(Utilities::FORMAT_DATE_DEFAULT_ONLY_DATE),
                'exception' => $exception,
                'exceptionDate' => $exceptionDate,
            ));
            $response->addDataset(new DataSet('calculationDate', $dataSet));
        }catch (\Exception $e){
            $response->addMessage(new Response\Message('Erro inesperado '.$e->getMessage()));
            throw $e;
        }
    }

    /**
     * @param $row
     * @return array
     */
    protected function getValuesCalculation($row)
    {
        $task = $row['task']?:null;
        $begin = $row['begin'];
        $end = $row['end'];
        $duration = $row['duration'];
        $parent = $row['parent'];
        $exception = $row['exception'];
        $exceptionDate = $row['exceptionDate'];
        $group = $row['group'];

        $begin = \DateTime::createFromFormat(Utilities::FORMAT_DATE_DEFAULT,$begin.' 00:00:00');
        $end = \DateTime::createFromFormat(Utilities::FORMAT_DATE_DEFAULT,$end.' 00:00:00');
        return array($task,$begin, $end, $duration, $parent, $exception, $exceptionDate,$group);
    }

    public function findMyProjectByUser(Request\Filter $request,Response $response){
        $conditions = $request->getFilterCriteria()->getConditions();
        $dataSet = InstanceProvider::getService()->getProject()->findMyProjectByUser($conditions);
        $response->addDataset(new DataSet('findMyProjectByUser', $dataSet));
    }

    public function findAllProjectByUser(Request\Filter $request,Response $response){
        $conditions = $request->getFilterCriteria()->getConditions();
        $projectId = Utilities::getValueInFilterDataSourceByName($conditions,'PROJECT');
        $projectId = $projectId?:0;
        $dataSet = InstanceProvider::getService()->getProject()->findAllProjectByUser($projectId);
        $response->addDataset(new DataSet('projects', $dataSet));
    }

    public function findMyProjectByUserWithMeeting(Request\Filter $request,Response $response){
        $conditions = $request->getFilterCriteria()->getConditions();
        $dataSet = InstanceProvider::getService()->getProject()->findMyProjectByUserWithMeeting($conditions);
        $response->addDataset(new DataSet('projects', $dataSet));
    }
    public function findMyProjectByUserInMeeting(Request\Filter $request,Response $response){
        $conditions = $request->getFilterCriteria()->getConditions();
        $userId = Utilities::getValueInFilterDataSourceByName($conditions,'MANAGER');
        $dataSet = InstanceProvider::getService()->getProject()->findMyProjectByUserInMeeting($userId);
        $response->addDataset(new DataSet('projects', $dataSet));
    }

    public function validateMaxDurationDay(Request\Filter $request,Response $response){
        $conditions = $request->getFilterCriteria()->getConditions();
        $taskId = empty($conditions[0]['value']) ? 0 : $conditions[0]['value'] ;
        $resourcesId = $conditions[1]['value'];
        $betweenDates = $conditions[2]['value'];
        $duration = $conditions[3]['value'];
        $project = $conditions[4]['value'];
        $precursorId = $conditions[5]['value'];
        $dataSet = InstanceProvider::getService()->getProject()->validateMaxDurationDay($taskId,$resourcesId,$betweenDates,$duration,$project,$precursorId);
        $response->addDataset(new DataSet('validateMaxDurationDay', $dataSet));
    }

    public function findAssociateProjectByUser(Request\Filter $request,Response $response){
        ini_set('max_execution_time', '60');
        set_time_limit(60);
        $conditions = $request->getFilterCriteria()->getConditions();
        $dataSet = InstanceProvider::getService()->getProject()->findAssociateProjectByUser($conditions);
        $response->addDataset(new DataSet('projects', $dataSet));
    }

    public function findProjectByUserPMA(Request\Filter $request,Response $response){
        $dataSet = InstanceProvider::getService()->getProject()->findProjectByUserPMA();
        $response->addDataset(new DataSet('projects', $dataSet));
    }

    public function parseHourOrDay($str){
        for($i = 0; $i < strlen($str); $i++){
            if($str[$i] == 'H' || $str[$i] == 'h'){
                preg_replace("/[^0-9,.]/","", $str);
                return $str;
            }
        }
        preg_replace("/[^0-9,.]/","", $str);
        return $str*8;
    }
    public function findProjectByOrganization(Request\Filter $request,Response $response){
        $dataSet = InstanceProvider::getService()->getProject()->findProjectByOrganization();
        $response->addDataset(new DataSet('projects', $dataSet));
    }
    public function findProjectByUser(Request\Filter $request,Response $response){
        $dataSet = InstanceProvider::getService()->getProject()->findProjectByUser();
        $response->addDataset(new DataSet('projects', $dataSet));
    }
    public function projectLog(Request\Filter $request,Response $response){
        $param = $request->getFilterCriteria()->getConditions();
        $projectId = $param[0]['value'];
        unset($param[0]);
        $dataSet = InstanceProvider::getService()->getProject()->findProjectLog($projectId,$param);
        $response->addDataset(new DataSet('projectLog', $dataSet));
    }

    public function findProjectParent(Request\Filter $request,Response $response)
    {
        ini_set('max_execution_time', '30');
        set_time_limit(30);
        $param = $request->getFilterCriteria()->getConditions();
        if (sizeof($param) > 0) {
            $projectId = Utilities::getValueInFilterDataSourceByName($param, 'PROJECT');
            $projectTypeId = Utilities::getValueInFilterDataSourceByName($param, 'PROJECT_TYPE');
            $filterColumn = Utilities::getFilterTableColumn($param, 'NAME');
            $dataSet = InstanceProvider::getService()->getProject()->findProjectParent($filterColumn);
            $response->addDataset(new DataSet('projects', $dataSet));
        }
    }

    public function findSponsorByProject(Request\Filter $request,Response $response)
    {
        $param = $request->getFilterCriteria()->getConditions();
        $projectId = Utilities::getValueInFilterDataSourceByName($param, 'PROJECT');
        if($projectId){
            $dataSet = InstanceProvider::getService()->getProject()->findSponsorByProject($projectId);
            $response->addDataset(new DataSet('findSponsorByProject', $dataSet));
        }

    }

    public function findManagerByProject(Request\Filter $request,Response $response)
    {
        $param = $request->getFilterCriteria()->getConditions();
        $projectId = Utilities::getValueInFilterDataSourceByName($param, 'PROJECT');
        if($projectId){
            $dataSet = InstanceProvider::getService()->getProject()->findManagerByProject($projectId);
            $response->addDataset(new DataSet('findManagerByProject', $dataSet));
        }

    }

    /**
     * Gets informations about a project.
     * @param Request\Filter $request
     * @param Response $response
     */
    public function basicProjectForm(Request\Filter $request, Response $response){
        try{
            $param = $request->getFilterCriteria()->getConditions();
            $projectId = $param[0]['value'];
            $resources = InstanceProvider::getService()->getProject()->getDataSetForEditProjectBasic($projectId);
            $response->addDataSet(new DataSet('basicProjectForm', $resources));

            InstanceProvider::getLog()->setLog(' + total de regras de negocio',false,3);
            InstanceProvider::getLog()->setLog(' = inicio - fim',false,4);
            $response->addDataset(new DataSet("log", InstanceProvider::getLog()->getLogRequest()));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function getDatesMinAndMaxByProjectId(Request\Filter $request, Response $response){
        $param = $request->getFilterCriteria()->getConditions();
        $projectId = Utilities::getValueInFilterDataSourceByName($param,'PROJECT');
        $resources = InstanceProvider::getService()->getProject()->getDatesMinAndMaxByProjectId($projectId);
        $response->addDataSet(new DataSet('getDatesMinAndMaxByProjectId', $resources));
    }

    public function modelProject(Request\Filter $request, Response $response){
        $param = $request->getFilterCriteria()->getConditions();
        $projectParent = Utilities::getValueInFilterDataSourceByName($param,'PROJECT_PARENT',true);
        $resources = InstanceProvider::getService()->getProject()->findModelProject($projectParent,$param);
        $response->addDataSet(new DataSet('modelProject', $resources));
    }

    public function modelProjectForFilter(Request\Filter $request, Response $response){
        $resources = InstanceProvider::getService()->getProject()->findModelProjectForFilter();
        $response->addDataSet(new DataSet('modelProject', $resources));
    }

    public function getBasicModelProject(Request\Filter $request, Response $response){
        try{
            $param = $request->getFilterCriteria()->getConditions();
            $projectId = Utilities::getValueInFilterDataSourceByName($param,'PROJECT');
            $resources = InstanceProvider::getService()->getProject()->getBasicModelProject($projectId);
            $response->addDataSet(new DataSet('getBasicModelProject', $resources));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function projectException(Request\Filter $request, Response $response){
        $param = $request->getFilterCriteria()->getConditions();
        $projectId = $param[0]['value'];
        $resources = InstanceProvider::getService()->getProject()->getDataSetForEditProjectException($projectId);
        $response->addDataSet(new DataSet('projectException', $resources));
    }

    public function fetchCompleteProjectInformation(Request\Filter $request,Response $response){
        $filter = $request->getFilterCriteria()->getConditions();
        $project = $filter[0]["value"];


        $dataBasic = InstanceProvider::getService()->getProject()->getDataSetForEditProjectBasic($project);
        $dataAdvanced = InstanceProvider::getService()->getProject()->getDataSetForEditProjectAdvanced($project);

        $response->addDataset(new DataSet("Basic", $dataBasic ));
        $response->addDataset(new DataSet("Advanced", $dataAdvanced));

    }

    public function parseHourOrDayEdit($str){
        $str=strtoupper($str);
        if(!(strpos($str,"D")=== false)){
            $str = $str*24;
        }
        return  $str = preg_replace("/[^0-9\s]/", "",$str);
    }

    public function editProjectBasic(Request\DataSet $request,Response $response){
        $data  = $request->getDataSet()->getRows();
        $param = $data[0];
        try {
            $projectId = $this->persistProject($param)->getId();
            $response->addDataset(new DataSet("ProjectId", array($projectId)));

            InstanceProvider::getLog()->setLog('inicio - fim',false,1);
            $response->addDataset(new DataSet("log", InstanceProvider::getLog()->getLogRequest()));
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function saveProduct(Request\DataSet $request,Response $response){
        $data  = $request->getDataSet()->getRows();
        $row = $data[0];
        try {
            $projectId = $row['ID'];
            $notifyIssueOpening = empty($row['NOTIFY_ISSUE_OPENING']) ? null : $row['NOTIFY_ISSUE_OPENING'];
            $productType = empty($row['PRODUCT_TYPE']) ? null : $row['PRODUCT_TYPE'];
            $allowReplicationLts = empty($row['ALLOW_REPLICATION_LTS']) ? null : $row['ALLOW_REPLICATION_LTS'];
            InstanceProvider::getService()->getProject()->saveProjectConfig($projectId, $notifyIssueOpening, $productType, $allowReplicationLts);
            $response->addDataset(new DataSet("ProjectId", array($projectId)));
            $response->addNotification(new Response\Notification(Messages::SUC_SUCCESSFUL_SAVED));
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public static function builderExceptionRow($row)
    {
        $recurrenceType = $row['RECURRENCE_TYPE'];
        $id = $row['ID'];
        $name = $row['NAME'];
        $beginTime = $row['TIME'];
        $status = empty($row['STATUS']) ? 'A' : $row['STATUS'];
        $beginDate = $row['BEGIN_DATE'];
        $occurrences = $row['OCCURRENCES'];
        $exceptionQuantitative = $row['EXCEPTION_QUANTITATIVE'];
        $exceptionDataEnd = $row['EXCEPTION_DATA_END'];
        $endDate = $row['END_DATE'];
        $sun = $row['SUN'];
        $mon = $row['MON'];
        $tue = $row['TUE'];
        $wed = $row['WED'];
        $thu = $row['THU'];
        $fri = $row['FRI'];
        $sat = $row['SAT'];
        $monthlyDay = $row['MONTHLY_DAY'];
        $monthly = $row['MONTHLY'];
        $yearDay = $row['YEAR_DAY'];
        $workTime = $row['WORKTIME'];
        $yearDayException = null;
        $weekDayException = null;
        $orderDay = null;
        $weekDay = null;
        $every = null;
        /**
         * Tratamentos de fields que mudou no frontend
         */
        if ($recurrenceType == \Service\CalendarDetail::RECURRENCE_TYPE_DAILY) {
            $every = $row['EVERY_DAY'];
        } else if ($recurrenceType == \Service\CalendarDetail::RECURRENCE_TYPE_WEEKLY) {
            $every = $row['EVERY_WEEK'];
        } else if ($recurrenceType == \Service\CalendarDetail::RECURRENCE_TYPE_MONTHLY) {
            $yearDayException = $row['YEAR_DAY_EXCEPTION_MONTH'];
            $weekDayException = $row['WEEK_DAY_EXCEPTION_MONTH'];
            if ($yearDayException == 'Y') {
                $monthlyDay = $row['MONTHLY_DAY'];
                $every = $row['EVERY_MONTH_YEAR'];
            } else {
                $orderDay = $row['ORDER_DAY_MONTH'];
                $weekDay = $row['WEEK_DAY_MONTH'];
                $every = $row['EVERY_MONTH_WEEK'];
            }
        } else if ($recurrenceType == \Service\CalendarDetail::RECURRENCE_TYPE_YEARLY) {
            $yearDayException = $row['YEAR_DAY_EXCEPTION_YEAR'];
            $weekDayException = $row['WEEK_DAY_EXCEPTION_YEAR'];
            if ($yearDayException == 'Y') {
                $yearDay = $row['YEAR_DAY'];
            } else {
                $orderDay = $row['ORDER_DAY_YEAR'];
                $weekDay = $row['WEEK_DAY_YEAR'];
                $monthly = $row['MONTHLY'];
            }
        }

        $beginDate = \DateTime::createFromFormat(Utilities::FORMAT_DATE_DEFAULT, $beginDate . ' 00:00:00');
        $endDate = \DateTime::createFromFormat(Utilities::FORMAT_DATE_DEFAULT, $endDate . ' 00:00:00');

        $yearDay = explode(' ', $yearDay);
        if (!empty($yearDay[1])) {
            $yearDay[1] = Utilities::$ARRAY_MONTH_FOR_NUMERIC[$yearDay[1]];
        }
        return array($name,$recurrenceType, $id, $status, $beginDate, $occurrences, $exceptionQuantitative, $exceptionDataEnd,
            $endDate, $every, $sun, $mon, $tue, $wed, $thu, $fri, $sat, $yearDayException, $monthlyDay, $monthly,
            $weekDayException, $orderDay, $weekDay, $yearDay, $workTime, $beginTime);
    }

    /**
     * @param $project
     * @return \Zeedhi\DevOps\Model\Projectmanagement\Entities\Project
     * @throws \Exception
     */
    public function persistProject($project){
        ini_set('max_execution_time','600');
        set_time_limit(600);
        try{
            $projectId = empty($project['ID']) ? null : $project['ID'];
            $parent    = empty($project['UPPER_PROJECT']) ? null : $project['UPPER_PROJECT'];
            $group     = $project['GROUP'];
            $name      = $project['NAME'];
            $status    = empty($project['STATUS']) ? 'A' : $project['STATUS'];
            $manager   = $project['MANAGER'];
            $sponsor   = $project['SPONSOR'];
            $mergeNewManager = empty($project['MERGE_NEW_MANAGER']) ? array() : $project['MERGE_NEW_MANAGER'];;
            $mergeSponsor = empty($project['MERGE_SPONSOR']) ? false : $project['MERGE_SPONSOR'];;
            $mergeOldManager = empty($project['MERGE_OLD_MANAGER']) ? array() : $project['MERGE_OLD_MANAGER'];
            $recursiveSponsor = empty($project['RECURSIVE_SPONSOR']) ? false : $project['RECURSIVE_SPONSOR'];
            $dateExceptionType = empty($project['DATE_EXCEPTION_ABBREVIATION']) ? 'MB' : $project['DATE_EXCEPTION_ABBREVIATION'];
            $dateException     = empty($project['DATE_EXCEPTION']) ? null : $this->createDateProject($project['DATE_EXCEPTION']);
            $beginDate = $this->createDateProject($project['START']);
            $endDate   = $this->createDateProject($project['END']);
            $projectType   = empty($project['PROJECT_TYPE_ID']) ? null : $project['PROJECT_TYPE_ID'];
            $description   = empty($project['DESCRIPTION']) ? null : $project['DESCRIPTION'];
            $category  = empty($project['PROJECT_CATEGORIES_ID']) ? null : $project['PROJECT_CATEGORIES_ID'];
            $projectStatus  = empty($project['PROJECT_STATUS_ID']) ? null : $project['PROJECT_STATUS_ID'];
            $projectModel  = empty($project['MODELS']) ? null : $project['MODELS'];
            $productVersionDevId  = empty($project['PRODUCT_VERSION_DEV_ID']) ? null : $project['PRODUCT_VERSION_DEV_ID'];
            $versionDev  = empty($project['VERSION_DEV']) ? null : $project['VERSION_DEV'];
            $mnemonicName = empty($project['MNEMONIC_NAME']) ? null : $project['MNEMONIC_NAME'];
            $isLtsReplication = empty($project['IS_LTS_REPLICATION']) ? null : $project['IS_LTS_REPLICATION'];
            if(!is_array($manager)){
                $manager = array($manager);
            }
            if($projectId) {
                $project = InstanceProvider::getService()->getProject()->editProject($projectId,$name,$manager,$sponsor,$status,$group,$parent,$beginDate,$endDate,
                    $dateExceptionType,$dateException,$mergeNewManager,$mergeSponsor,$description,$projectType,$category,$projectStatus,
                    $mergeOldManager, $recursiveSponsor, $productVersionDevId, $isLtsReplication
                );
            } else {
                $project = InstanceProvider::getService()->getProject()->newProject($name, $parent, $status, 1, $endDate, $beginDate, $group, $manager,
                    $sponsor, $dateExceptionType, $dateException,$projectType, $category, $description,$projectStatus,$projectModel, $productVersionDevId, $mnemonicName, $isLtsReplication);
            }
            return $project;
        }
        catch(\Exception $e){
            throw $e;
        }
    }

    /**
     * @param $date
     * @return \DateTime
     */
    protected function createDateProject($date)
    {
        return strpos($date, '-') === false
            ? \DateTime::createFromFormat(Utilities::FORMAT_DATE_DEFAULT_ONLY_DATE,$date)
            : new \DateTime($date);
    }

    public function allocateResourceProject (Request\DataSet $request, Response $response) {
        $data = $request->getDataSet()->getRows();
        $param = $data[0];
        $project = $param['PROJECT'];
        try {
            foreach($param['RESOURCES_ID'] as $resource){
                InstanceProvider::getService()->getResourceProject()->createResourceProject($resource, $project, null, false);
            }
        }catch(\Exception $e){
            throw $e;
        }
    }

    public function editProjectResource(Request\DataSet $request,Response $response){
        $data = $request->getDataSet()->getRows();
        $param = $data[0];
        $resources = $param['RESOURCES'];
        $project = $param['PROJECT'];
        $recursive = empty($param['recursive']) ? false : $param['recursive'];
        foreach($resources as $resource){
            InstanceProvider::getService()->getResourceProject()->createResourceProject($resource['ID'], $project, null, $recursive);
        }
        $dataResource = InstanceProvider::getService()->getResource()->getDataSetForEditProjectResource($project);
        $response->addDataset( new DataSet("Resource", $dataResource ));
        $response->addNotification(new Response\Notification('Recurso adicionado com sucesso.'));
    }

	public function deleteProjectResource(Request\DataSet $request,Response $response){
        $data = $request->getDataSet()->getRows();
        $param = $data[0];
        $recursive = empty($param['recursive']) ? false : $param['recursive'];
        $errorMessages = array();
        try{
            foreach($param['CHECKED_ROWS'] as $row){
                $errorMessages = array_merge(   $errorMessages,
                                                InstanceProvider::getService()->getResourceProject()->deleteResourceProjectById($row['ID_BASE'], $recursive));
            }
            if(!empty($errorMessages)) {
                $response->addMessage(new Response\Message(Messages::ERR_NOT_POSSIBLE_TO_DELETE_RESOURCE_MANAGER_FROM_PROJECT,
                    'error'));
            }else{
                $response->addNotification(new Response\Notification('Recurso removido com sucesso.'));
            }

        } catch (\Exception $e){
            $response->addMessage(new Response\Message($e->getMessage(),Response\Message::TYPE_WARNING));
        }
    }

    public function deleteProject(Request\DataSet $request,Response $response){
        $data = $request->getDataSet()->getRows();
        $param = $data[0];
        try{
            InstanceProvider::getService()->getProject()->deleteProject($param['ID']);
            $response->addNotification(new Response\Notification('Projeto apagado com sucesso.'));
        }catch (\Exception $e){
           throw $e;
        }
    }

    /*********************************************************************
     * Find all projects that are managed by the same manager.
     * @param Request\DataSet $request
     * @param Response $response
     *********************************************************************/
	public function findProjectsSameManager(Request\Filter $request, Response $response) {
		$filter = $request->getFilterCriteria()->getConditions();
		$projectExclusionId = $filter[0]['value'];
        $projectExclusionStatus = empty($filter[1]) ? array() : $filter[1]['value'];
		try {
			$tasksMoved = InstanceProvider::getService()->getProject()->getProjectsByManager($projectExclusionId, $projectExclusionStatus);
			$response->addDataSet(new DataSet("projectSameManager", $tasksMoved));
		} catch (\Exception $e) {
			$response->setStatus(Response::STATUS_ERROR);
			$response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_WARNING));
		}
	}
    public function findProjectsMnemonics(Request\Filter $request, Response $response) {
        $conditions = $request->getFilterCriteria()->getConditions();
        $valueId = Utilities::getValueInFilterDataSourceByName($conditions, 'ID');
        $isCopyTask = Utilities::getValueInFilterDataSourceByName($conditions, 'isCopyTask');
        if($valueId) {
            if ($isCopyTask) {
                $tasksId = '';
                foreach ($valueId as $taskId) {
                    $tasksId = empty($tasksId) ? $taskId['ID'] : $tasksId . ',' . $taskId['ID'];
                }
                $valueId = $tasksId;
            }
        $mnemonics = InstanceProvider::getService()->getProject()->findProjectsMnemonics($valueId, $isCopyTask);
        }else {
            $mnemonics = [];
        }
        $response->addDataSet(new DataSet("projectHasMnemonic", $mnemonics));

	}

    /*********************************************************************
     * Moves a task between projects and/or tasks.
     * @param Request\DataSet $request
     * @param Response $response
     *********************************************************************/
	public function moveTask(Request\DataSet $request, Response $response) {
		$dataSet                = $request->getDataSet()->getRows();
		$targetProject          = $dataSet[0]['targetProject'];
		$originProject          = $dataSet[3]['projectId'];
		$targetTask             = empty($dataSet[1]['targetTask']) ? null : $dataSet[1]['targetTask'];
        $tasks                  = $dataSet[2];
        $addDifferentResources  = $dataSet[4]['addDifferentResources'];
        $differentResourcesIds  = empty($dataSet[5]['differentResourcesIds']) ? null : $dataSet[5]['differentResourcesIds'];
        $replanningReasonId     = empty($dataSet[6]['replanningReasonId']) ? null : $dataSet[6]['replanningReasonId'];
        $taskIdsNotMoved       = empty($dataSet[7]['taskIdsNotMoved']) ? null : $dataSet[7]['taskIdsNotMoved'];
        try {

            $directoryBasicUpload = InstanceProvider::getEnvironment()->getDirectoryBasicUpload();
            InstanceProvider::getService()->getProject()->moveTask($targetProject, $targetTask, $tasks,
                $directoryBasicUpload, $addDifferentResources,
                $differentResourcesIds, $replanningReasonId, $originProject, $taskIdsNotMoved);
			$response->addNotification(new Response\Notification('Tarefa(s) movida(s) com sucesso.'));
		} catch (\Exception $e) {
			$response->setError(new Response\Error($e->getMessage(), $e->getCode()));
		}
	}


    public function copyTask(Request\DataSet $request, Response $response) {
		$dataSet                = $request->getDataSet()->getRows();
		$targetProject          = $dataSet[0]['targetProject'];
		$targetTask             = empty($dataSet[1]['targetTask']) ? null : $dataSet[1]['targetTask'];
		$tasks                  = $dataSet[2];
        $addDifferentResources  = $dataSet[4]['addDifferentResources'];
        $differentResourcesIds  = empty($dataSet[5]['differentResourcesIds']) ? null : $dataSet[5]['differentResourcesIds'];
        $useMnemonic            = empty($dataSet[6]['useMnemonic']) ? null : $dataSet[6]['useMnemonic'];
		try {
            $directoryBasicUpload = InstanceProvider::getEnvironment()->getDirectoryBasicUpload();
			InstanceProvider::getService()->getProject()->copyTask($targetProject, $targetTask, $tasks, $directoryBasicUpload, false,
                $addDifferentResources, $differentResourcesIds, false, null, null, null, null, null, null, $useMnemonic);
			$response->addNotification(new Response\Notification('Tarefa(s) copiada(s) com sucesso.'));
        } catch (\Exception $e) {
			throw $e;
		}
	}
    public function saveRepository(Request\DataSet $request, Response $response){
        $param = $request->getDataSet()->getRows();
        $param = $param[0];
        $repository = $param['REPOSITORY'];
        $status = isset($param['STATUS']) ? $param['STATUS'] : 'A';
        $url = $param['URL'];
        $description = empty($param['DESCRIPTION']) ? '' : $param['DESCRIPTION'];
        $project = $param['PROJECT'];
        if(isset($param['REPOSITORY_ID'])){
            $repositoryId = $param['REPOSITORY_ID'];
        }else{
            $repositoryId = 0;
        }

        try {
            $repository = InstanceProvider::getService()->getRepository()->saveRepository($repository, $status, $url, $description, $repositoryId, $project);
            $response->addDataSet(new DataSet("saveRepository", array($repository)));
            $response->addNotification(new Response\Notification('Repositório adicionado com sucesso.'));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_WARNING));
        }
    }


    public function saveProjectPaths(Request\DataSet $request, Response $response){
        $param = $request->getDataSet()->getRows();
        $param = $param[0];
        $repository = isset($param['REPOSITORY']) ? $param['REPOSITORY'] : null;
        $repositoryStatus = isset($param['REPOSITORY_STATUS']) ? $param['REPOSITORY_STATUS'] : null;
        $repositoryId = isset($param['REPOSITORY_ID']) ? $param['REPOSITORY_ID'] : null;

        $resourceProjectId = isset($param['RESOURCE_PROJECT']) ? $param['RESOURCE_PROJECT'] : null;
        $projectId = isset($param['PROJECT_ID']) ? $param['PROJECT_ID'] : null;

        $directory = isset($param['DIRECTORY']) && !empty($param['DIRECTORY']) ? $param['DIRECTORY'] : null;
        $taskId = isset($param['TASK']) ? $param['TASK'] : null;
        try
        {
            $paths = InstanceProvider::getService()->getResourceProject()->setResourceProjectPaths($repositoryId, $directory, $resourceProjectId,
                                                                            $projectId, $repositoryStatus, $taskId);
            $response->addDataSet(new DataSet("saveResourceProjectPaths", $paths));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_WARNING));
        }
    }

    public function findProducts(Request\Filter $request, Response $response) {
        $param = $request->getFilterCriteria()->getConditions();
        $filterName = false;
        $filterOperator = false;
        $filterValue = false;

        if(empty($param[0]) ){
            $projectId = 0;
        }else if($param[count($param)-1]["operator"] == "LIKE_ALL"){
            if(count($param)-1 == 0){
                $projectId = 0;
            }
            $filterName = "UPPER(".$param[count($param)-1]["columnName"].")";
            $filterOperator = "LIKE";
            $filterValue = "UPPER(".$param[count($param)-1]["value"].")";
        }else {
            $projectId = $param[0]['value'];
        }
        $products = InstanceProvider::getService()->getProject()->findProducts($projectId, $filterName, $filterOperator, $filterValue);
        $response->addDataSet(new DataSet('products', $products));
    }

    public function findProductsExceptConcludedOrCanceled(Request\Filter $request, Response $response) {
        $conditions = $request->getFilterCriteria()->getConditions();
        $page = $request->getFilterCriteria()->getPage();
        $pageSize = $request->getFilterCriteria()->getPageSize();
        $products = InstanceProvider::getService()->getProject()->findProductsExceptConcludedOrCanceled($conditions, $page, $pageSize);

        $response->addDataSet(new DataSet('products', $products));
    }

    public function findAllProductsBudget(Request\Filter $request, Response $response) {
        $conditions = $request->getFilterCriteria()->getConditions();
        $page = $request->getFilterCriteria()->getPage();
        $pageSize = $request->getFilterCriteria()->getPageSize();
        $products = InstanceProvider::getService()->getProject()->findAllProductsBudget($conditions, $page, $pageSize);

        $response->addDataSet(new DataSet('products', $products));
    }

    public function findMyProducts(Request\Filter $request, Response $response)
    {
        $conditions = $request->getFilterCriteria()->getConditions();
        $structures = DevopsUtilities::getValueInFilterDataSourceByName($conditions, 'STRUCTURE', true);
        $products = InstanceProvider::getService()->getProject()->findMyProducts(0, $conditions, $structures);
        $response->addDataSet(new DataSet('findMyProducts', $products));
    }

    public function findProjectProductAndMeeting(Request\Filter $request, Response $response) {
        $filters = $request->getFilterCriteria()->getConditions();
        $products = InstanceProvider::getService()->getProject()->findProjectProductAndMeeting($filters);
        $response->addDataSet(new DataSet('products', $products));
    }

    public function findAllProducts(Request\Filter $request, Response $response) {

        $products = InstanceProvider::getService()->getProject()->findAllProducts();
        $response->addDataSet(new DataSet('products', $products));
    }

    public function getBoard(Request\Filter $request, Response $response) {
        $filter = $request->getFilterCriteria()->getConditions();
        $board = $filter[0]['value'];
        try {
            $board = InstanceProvider::getService()->getProject()->getBoardData($board);
            $response->addDataSet(new DataSet('board', $board));
        }
        catch(\Exception $e){
            $response->addMessage(new Response\Message('Não foi possível carregar os dados de sua board, pois ' . $e->getMessage()));
        }
    }

    public function saveNewBoard(Request\DataSet $request, Response $response) {
        $dataSet = $request->getDataSet()->getRows();
        $param = $dataSet[0];

        $description = empty($param["DESCRIPTION"]) ? "" : $param["DESCRIPTION"];

        $name = $param['NAME'];
        $manager = $param['MANAGER'];
        $structure = $param['GROUP'];
        $parent = empty($param['UPPER_PROJECT']) ? null : $param['UPPER_PROJECT'];

        try{
            InstanceProvider::getService()->getProject()->saveNewBoard($name, $manager, $structure, $parent, $description);
            $response->addMessage(new Response\Message("Novo board criado com sucesso"));
        }
        catch(\Exception $e) {
            $response->addMessage(new Response\Message("Não foi possível criar novo board, pois " . $e->getMessage()));
        }
    }

    public function saveBoard(Request\DataSet $request, Response $response) {
        $dataSet = $request->getDataSet()->getRows();
        $param = $dataSet[0];

        $id = $param['ID'];
        $description = empty($param["DESCRIPTION"]) ? "" : $param["DESCRIPTION"];
        $name = $param['NAME'];
        $manager = $param['MANAGER'];
        $structure = $param['STRUCTURE'];
        $parent = empty($param['UPPER_PROJECT']) ? null : $param['UPPER_PROJECT'];

        try{
            InstanceProvider::getService()->getProject()->saveBoard($id, $name, $manager, $structure, $parent, $description);
            $response->addMessage(new Response\Message("Board atualizado com sucesso"));
        }
        catch(\Exception $e) {
            $response->addMessage(new Response\Message("Não foi possível atualizar board, pois " . $e->getMessage()));
        }
    }

    public function findTopMostManagerStructure(Request\Filter $request, Response $response){
        $filter = $request->getFilterCriteria()->getConditions();
        $userId = $filter[0]['value'];
        $structures = $filter[1]['value'];
        try {
            $structure = InstanceProvider::getService()->getProject()->getTopMostManagerStructure($userId, $structures);
            $response->addDataSet(new DataSet("structures", array($structure)));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_WARNING));
        }
    }

    public function isMergeRequestProject(Request\DataSet $request, Response $response) {
        $row = $request->getDataSet()->getRows();
        $param = $row[0];
        $project = $param['PROJECT'];
        try {
            $isMergeRequestProject = InstanceProvider::getService()->getProject()->isMergeRequestProject($project);
            $response->addDataset(new DataSet("isMergeRequestProject", array($isMergeRequestProject)));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_WARNING));
        }
    }


    public function findProjectCategories(Request\Filter $request, Response $response){
        $filters = $request->getFilterCriteria()->getConditions();
        $projectType = null;
        try {
            foreach($filters as $filter) {
                if (isset($filter["columnName"]) && $filter["columnName"] == "PROJECT_TYPE") {
                        $projectType = $filter["value"];
                }
            }
            $filterLabel = Utilities::getFilterTableColumn($filters, 'LABEL');
            $projectCategories = InstanceProvider::getService()->getProject()->findProjectCategories($projectType, $filterLabel);
            $response->addDataSet(new DataSet('ProjectCategories', $projectCategories));
        } catch (\Exception $e){
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_WARNING));
        }
    }

    public function findProjectTypes(Request\Filter $request, Response $response) {
        try {
            $filter = $request->getFilterCriteria()->getConditions();
            $filterName = !empty($filter) && isset($filter[1]['columnName']) ? $filter[1]['columnName'] : false;
            $filterOperator = !empty($filter) && isset($filter[1]['operator']) ? "LIKE" : false;
            $filterValue = !empty($filter) && isset($filter[1]['value']) ? $filter[1]['value'] : false;
            $projectTypes = InstanceProvider::getService()->getProject()->findProjectTypesFilter($filterName, $filterOperator, $filterValue);
            $response->addDataset(new DataSet('findProjectTypes', $projectTypes));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_WARNING));
        }
    }

    public function findProjectTypesToIssue(Request\Filter $request, Response $response) {
        try {
            $filter = $request->getFilterCriteria()->getConditions();
            $projectTypes = InstanceProvider::getService()->getProject()->findProjectTypesToIssue($filter);
            $response->addDataset(new DataSet('findProjectTypes', $projectTypes));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_WARNING));
        }
    }

    /**
     * Gets all project status.
     * @param Request\Filter $request
     * @param Response $response
     */
    public function findProjectStatus(Request\Filter $request, Response $response)
    {
        try {
            $filter = $request->getFilterCriteria()->getConditions();
            $filterName = false;
            $filterValue = false;
            $filterOperator = false;
            if(count($filter) > 0 && $filter[count($filter)-1]['operator'] == "LIKE_ALL"){
                $filterName = $filter[count($filter)-1]['columnName'];
                $filterValue = $filter[count($filter)-1]['value'];
                $filterOperator = "LIKE";
            }
            $projectStatus = InstanceProvider::getService()->getProject()->findProjectStatus($filterName, $filterOperator, $filterValue, $filter);
            $response->addDataSet(new DataSet('projectStatus', $projectStatus));
        } catch (\Exception $error) {
            $response->addMessage(new Response\Message(\Util\Messages::ERR_NOT_SUCCESSFUL_FOUND_EXPLANATION . $error->getMessage()));
        }
    }

    public function findProjectStatusInEditProject(Request\Filter $request, Response $response)
    {
        try {
            $filters = $request->getFilterCriteria()->getConditions();
            $projectStatus = InstanceProvider::getService()->getProject()->findProjectStatusInEditProject($filters);
            $response->addDataSet(new DataSet('projectStatus', $projectStatus));
        } catch (\Exception $error) {
            $response->addMessage(new Response\Message(\Util\Messages::ERR_NOT_SUCCESSFUL_FOUND_EXPLANATION . $error->getMessage()));
        }
    }

    /**
     * Saves a project status, does not matter if
     * is a new one or an already existent.
     * @param Request\DataSet $request
     * @param Response $response
     */
    public function saveProjectStatus(Request\DataSet $request, Response $response){
        try {
            $projectStatus = $request->getDataSet()->getRows()[0];
            $statusId       = !empty($projectStatus['ID']) ? $projectStatus['ID'] : null;
            $statusLabel    = $projectStatus['LABEL'];
            $statusGeneratesLog = $projectStatus['GENERATES_LOG'];
            $statusTaskCanStart = $projectStatus['TASK_CAN_START'];
            $control = $projectStatus['CONTROL']?:null;

            // Updates an already existent
            if(!empty($projectStatus['ID'])){
                InstanceProvider::getService()->getProject()->updateProjectStatus($statusId, $statusLabel, $statusGeneratesLog, $statusTaskCanStart,$control);
                $response->addNotification(new Response\Notification(\Util\Messages::SUC_SUCCESSFUL_UPDATED));
            }
            // Saves a new one
            else {
                $savedRow = InstanceProvider::getService()->getProject()->createProjectStatus($statusLabel, $statusGeneratesLog, $statusTaskCanStart,$control);
                $dataSet[] = array('ID' => $savedRow->getId());
                $response->addDataSet( new DataSet('projectStatus',$dataSet));
                $response->addNotification(new Response\Notification(\Util\Messages::SUC_SUCCESSFUL_CREATED));
            }
        }
        catch(\Exception $error){
            throw $error;
        }
    }

    /**
     * Remove all selected project status.
     * @param Request\DataSet $request
     * @param Response $response
     */
    public function removeProjectStatus(Request\DataSet $request, Response $response){
        $dataSet = $request->getDataSet()->getRows()[0];
        try {
            foreach($dataSet['CHECKED_ROWS'] as $row){
                InstanceProvider::getService()->getProject()->removeProjectStatus($row['ID']);
            }
            $response->addMessage(new Response\Message(\Util\Messages::SUC_SUCCESSFUL_REMOVED));
        } catch(\Exception $error){
            $response->addMessage(new Response\Message(\Util\Messages::ERR_NOT_SUCCESSFUL_DELETED_EXPLANATION . $error->getMessage()));
        }
    }

    public function singleRemoveProjectStatus(Request\DataSet $request, Response $response){
        $dataSet = $request->getDataSet()->getRows()[0];
        try {
            InstanceProvider::getService()->getProject()->removeProjectStatus($dataSet['ID']);
            $response->addNotification(new Response\Notification(\Util\Messages::SUC_SUCCESSFUL_REMOVED));
        } catch(\Exception $error){
            $response->addMessage(new Response\Message(\Util\Messages::ERR_NOT_SUCCESSFUL_DELETED_EXPLANATION . $error->getMessage()));
        }
    }

    public function BasicCalendarSave(Request\DataSet $request, Response $response)
    {
        $row = $request->getDataSet()->getRows()[0];
        $calendarId = $row["ID"];
        $projectId = $row["PROJECT"];
        $calendarDefaultId = $row['CALENDAR_DEFAULT'];
        $isApplyChildrenProject = $row["IS_APPLY_CHILDREN_PROJECT"];
        $calendar = InstanceProvider::getService()->getProject()->saveEditCalendar($calendarId ,$calendarDefaultId, $projectId, $isApplyChildrenProject);
        $response->addDataSet(new DataSet('basicCalendarStructure', [$calendar->getId()]));

    }

    public function findBasicCalendar(Request\Filter $request, Response $response)
    {
        $filter = $request->getFilterCriteria()->getConditions();

        $project = Utilities::getValueInFilterDataSourceByName($filter,'PROJECT');
        $calendars = InstanceProvider::getService()->getProject()->findBasicCalendar($project);
        $response->addDataSet(new DataSet('findBasicCalendar',($calendars)));

    }
    public function BasicCalendarResourceSave(Request\DataSet $request, Response $response)
    {
        $row = $request->getDataSet()->getRows()[0];

        $id = $row["ID"];
        $calendarDefault = $row["CALENDAR_DEFAULT"];
        $projectResource = $row["PROJECT_RESOURCE"];
        $calendar = InstanceProvider::getService()->getProject()->saveEditCalendarResource($id,$calendarDefault,$projectResource);
        $response->addDataSet(new DataSet('basicCalendarStructure', [$calendar->getId()]));

    }

    public function findBasicCalendarResource(Request\Filter $request, Response $response)
    {
        $filter = $request->getFilterCriteria()->getConditions();

        $projectResource = Utilities::getValueInFilterDataSourceByName($filter,'PROJECT_RESOURCE');
        $calendars = InstanceProvider::getService()->getProject()->findBasicCalendarResource($projectResource);
        $response->addDataSet(new DataSet('findBasicCalendar',($calendars)));

    }

    public function findCalendarDatesProject(Request\Filter $request, Response $response)
    {
        $filters = $request->getFilterCriteria()->getConditions();
        $month = Utilities::getValueInFilterDataSourceByName($filters,'MONTH');
        $year = Utilities::getValueInFilterDataSourceByName($filters,'YEAR');
        $projectId = Utilities::getValueInFilterDataSourceByName($filters,'PROJECT');
        $dataSet = array();
        list($beginDate, $endDate) = Utilities::getBeginAndEndDateByMonth($month,$year);
        if($projectId){
            $dataSet = InstanceProvider::getService()->getProject()->findExceptionCalendarDatesProject($projectId,$beginDate, $endDate);
        }
        $response->addDataset(new DataSet('CalendarDetail', $dataSet));
    }
    public function findCalendarDatesProjectResource(Request\Filter $request, Response $response)
    {
        $filters = $request->getFilterCriteria()->getConditions();
        $month = Utilities::getValueInFilterDataSourceByName($filters,'MONTH');
        $year = Utilities::getValueInFilterDataSourceByName($filters,'YEAR');
        $projectResourceId = Utilities::getValueInFilterDataSourceByName($filters,'PROJECT_RESOURCE');
        $dataSet = array();
        list($beginDate, $endDate) = Utilities::getBeginAndEndDateByMonth($month,$year);
        if($projectResourceId){
            $dataSet = InstanceProvider::getService()->getProject()->findExceptionCalendarDatesProjectResource($projectResourceId,$beginDate, $endDate);
        }
        $response->addDataset(new DataSet('CalendarDetail', $dataSet));
    }

    public function findByProjectCalendarDetail(Request\Filter $request, Response $response)
    {
        $filters = $request->getFilterCriteria()->getConditions();
        $id = Utilities::getValueInFilterDataSourceByName($filters,'ID', true);
        $projectId = Utilities::getValueInFilterDataSourceByName($filters,'PROJECT', true);
        $filter = DevopsUtilities::createQueryFiltered($filters);
        $dataSet = array();
        if($projectId){
            $dataSet = InstanceProvider::getService()->getProject()->findCalendarExceptionByProject($projectId,$id, $filter);
        }
        $response->addDataset(new DataSet('CalendarDetailProject', $dataSet));
    }
    public function findByProjectResourceCalendarDetail(Request\Filter $request, Response $response)
    {
        $filters = $request->getFilterCriteria()->getConditions();
        $id = Utilities::getValueInFilterDataSourceByName($filters,'ID');
        $projectResourceId = Utilities::getValueInFilterDataSourceByName($filters,'PROJECT_RESOURCE');
        $dataSet = array();
        if($projectResourceId){
            $dataSet = InstanceProvider::getService()->getProject()->findCalendarExceptionByProjectResource($projectResourceId,$id);
        }
        DevOpsUtilities::setDataSetDefault($request, $response, $dataSet);
    }
    public function findCalendarWeeklyDays(Request\Filter $request, Response $response)
    {
        $filters = $request->getFilterCriteria()->getConditions();
        $id = Utilities::getValueInFilterDataSourceByName($filters,'ID', true);
        $projectId = Utilities::getValueInFilterDataSourceByName($filters,'PROJECT', true);
        $filter = DevopsUtilities::createQueryFiltered($filters);
        $dataSet = array();
        if($projectId){
            $dataSet = InstanceProvider::getService()->getProject()->findCalendarWeeklyDays($projectId,$id, $filter);
        }
        DevOpsUtilities::setDataSetDefault($request, $response, $dataSet);
    }
    public function findCalendarProjectResourceWeeklyDays(Request\Filter $request, Response $response)
    {
        $filters = $request->getFilterCriteria()->getConditions();
        $id = Utilities::getValueInFilterDataSourceByName($filters,'ID');
        $projectResourceId = Utilities::getValueInFilterDataSourceByName($filters,'PROJECT_RESOURCE');
        $dataSet = array();
        if($projectResourceId){
            $dataSet = InstanceProvider::getService()->getProject()->findCalendarProjectResourceWeeklyDays($projectResourceId,$id);
        }
        DevOpsUtilities::setDataSetDefault($request, $response, $dataSet);
    }

    public function saveWeeklyDays(Request\DataSet $request, Response $response)
    {
        try{
            $row = $request->getDataSet()->getRows()[0];

            list($calendar, $id, $typeSunday, $dailyHoursSunday, $typeMonday, $dailyHoursMonday,
                $typeTuesday, $dailyHoursTuesday, $typeWednesday, $dailyHoursWednesday,
                $typeThursday, $dailyHoursThursday, $typeFriday, $dailyHoursFriday,
                $typeSaturday, $dailyHoursSaturday, $beginDate, $endDate, $name, $projectId, $isApplyChildrenProject) =
                CalendarWeeklyDays::buidingCreateWeekWork($row);
            $dataSet = InstanceProvider::getService()->getProject()->beforeSaveWeeklyDays($id, $calendar, $name, $typeSunday, $dailyHoursSunday,$typeMonday,
                $dailyHoursMonday, $typeTuesday, $dailyHoursTuesday, $typeWednesday, $dailyHoursWednesday, $typeThursday,
                $dailyHoursThursday, $typeFriday, $dailyHoursFriday, $typeSaturday, $dailyHoursSaturday, $beginDate, $endDate,
                $projectId, $isApplyChildrenProject);

            $response->addDataset(new DataSet('calendarWeeklyDaysProject', array("ID" => $dataSet->getId())));
            $response->addNotification(new Response\Notification(\Util\Messages::SUC_SUCCESSFUL_SAVED));
        }catch(\Exception $e){
            $response->addMessage(new Response\Message('Erro inesperado :'.$e->getMessage(),Response::STATUS_ERROR));
        }
    }

    public function saveWeeklyDaysResource(Request\DataSet $request, Response $response)
    {
        try{
            $row = $request->getDataSet()->getRows()[0];

            list($calendar, $id, $typeSunday, $dailyHoursSunday, $typeMonday, $dailyHoursMonday,
                $typeTuesday, $dailyHoursTuesday, $typeWednesday, $dailyHoursWednesday,
                $typeThursday, $dailyHoursThursday, $typeFriday, $dailyHoursFriday,
                $typeSaturday, $dailyHoursSaturday, $beginDate, $endDate, $name, $projectId, $isApplyChildrenProject) =
                CalendarWeeklyDays::buidingCreateWeekWork($row);
            $dataSet = InstanceProvider::getService()->getProject()->beforeSaveWeeklyDays($id, $calendar, $name, $typeSunday, $dailyHoursSunday,$typeMonday,
                $dailyHoursMonday, $typeTuesday, $dailyHoursTuesday, $typeWednesday, $dailyHoursWednesday, $typeThursday,
                $dailyHoursThursday, $typeFriday, $dailyHoursFriday, $typeSaturday, $dailyHoursSaturday, $beginDate, $endDate,
                $projectId, $isApplyChildrenProject);

            $response->addDataset(new DataSet('calendarWeeklyDaysProjectResource', array("ID" => $dataSet->getId())));
            $response->addNotification(new Response\Notification(\Util\Messages::SUC_SUCCESSFUL_SAVED));
        }catch(\Exception $e){
            $response->addMessage(new Response\Message('Erro inesperado :'.$e->getMessage(),Response::STATUS_ERROR));
        }
    }

    /************************************************************
     * Gets different resources between project and task
     * @param Request\Filter    $request
     * @param Response          $response
     ************************************************************/
    public function getDifferentResourcesBetweenProjectAndTask(Request\Filter $request, Response $response){
        try {
            $filter         = $request->getFilterCriteria()->getConditions();
            $tasksIds       = $filter[0]['value'];
            $projectId      = $filter[1]['value'];
            $diffResources  = InstanceProvider::getService()->getProject()->getDifferentResourcesBetweenProjectAndTask($tasksIds, $projectId);
            $response->addDataSet(new DataSet('differentResources', $diffResources));
        } catch(\Exception $e){
            $response->addMessage(new Response\Message('Ocorreu um erro ao tentar obter os recursos que estão
            nas tarefas mas não estão no projeto: ' . $e->getMessage(), Response::STATUS_ERROR));
        }
    }

    public function findProjectStatusByControl(Request\Filter $request, Response $response)
    {
        try {
            $filter = $request->getFilterCriteria()->getConditions();
            $projectStatus = InstanceProvider::getService()->getProject()->findProjectStatusByControl($filter[0]['value']);
            $response->addDataSet(new DataSet('projectStatus', $projectStatus));
        } catch (\Exception $error) {
            $response->addMessage(new Response\Message(\Util\Messages::ERR_NOT_SUCCESSFUL_FOUND_EXPLANATION . $error->getMessage()));
        }
    }

    public function validateRepositoriesMoveTask(Request\Filter $request, Response $response)
    {
        try {
            $filters = $request->getFilterCriteria()->getConditions();
            $tasks = Utilities::getValueInFilterDataSourceByName($filters,'TASKS');
            $projectId = Utilities::getValueInFilterDataSourceByName($filters,'PROJECT');
            $repositories = InstanceProvider::getService()->getProject()->validateRepositoriesMoveTask($tasks, $projectId);
            $response->addDataSet(new DataSet('repositoriesNotFound', $repositories));
        } catch (\Exception $error) {
            $response->addMessage(new Response\Message($error->getMessage()));
        }
    }

    public function validateConcludedTaskToMove(Request\Filter $request, Response $response)
    {
        try {
            $filters = $request->getFilterCriteria()->getConditions();
            $tasks = Utilities::getValueInFilterDataSourceByName($filters, 'TASKS');
            $projectId = Utilities::getValueInFilterDataSourceByName($filters, 'PROJECT_ID');
            $targetProjectId = Utilities::getValueInFilterDataSourceByName($filters, 'TARGET_PROJECT_ID');
            $data = InstanceProvider::getService()->getProject()->validateConcludedTaskToMove($tasks, $projectId, $targetProjectId);
            $response->addDataSet(new DataSet('validateConcludedTaskToMove', $data));
        } catch (\Exception $error) {
            $response->addMessage(new Response\Message($error->getMessage()));
        }
    }

     public function findAllProjects(Request\Filter $request, Response $response)
    {
        try {
            $filters = $request->getFilterCriteria()->getConditions();
            $page = $request->getFilterCriteria()->getPage();
            $pageSize = $request->getFilterCriteria()->getPageSize();
            $projects = InstanceProvider::getService()->getProject()->findAllProjects($filters, $page, $pageSize);
            $response->addDataSet(new DataSet('findAllProject', $projects));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function hasErrorTypeChildrenProjects(Request\Filter $request, Response $response)
    {
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $data = InstanceProvider::getService()->getProject()->hasErrorTypeChildrenProjects($conditions);
            $response->addDataSet(new DataSet('hasErrorTypeChildrenProjects', [$data]));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function projectHasTask(Request\Filter $request, Response $response)
    {
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $projectId = DevOpsUtilities::getValueInFilterDataSourceByName($conditions, 'PROJECT_ID', true);
            $data = InstanceProvider::getService()->getProject()->projectHasTask($projectId, $conditions);
            $response->addDataSet(new DataSet('projectHasTask', [$data]));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function hasProjectWithLtsSubReleaseFinalized(Request\Filter $request, Response $response)
    {
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $projectId = DevOpsUtilities::getValueInFilterDataSourceByName($conditions, 'PROJECT_ID');
            $data = InstanceProvider::getService()->getProject()->hasProjectWithLtsSubReleaseFinalized($projectId);
            $response->addDataSet(new DataSet('hasProjectWithLtsSubReleaseFinalized', [$data]));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function hasErrorTypeChildrenProjectsAndHasProjectWithLtsSubReleaseFinalized(Request\Filter $request, Response $response)
    {
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $projectId = DevOpsUtilities::getValueInFilterDataSourceByName($conditions, 'PROJECT_ID');
            $data = InstanceProvider::getService()->getProject()->hasErrorTypeChildrenProjectsAndHasProjectWithLtsSubReleaseFinalized($projectId);
            $response->addDataSet(new DataSet('hasErrorTypeChildrenProjectsAndHasProjectWithLtsSubReleaseFinalized', [$data]));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function validateParentProjectChoiceByUser(Request\Filter $request, Response $response)
    {
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $projectId = DevOpsUtilities::getValueInFilterDataSourceByName($conditions, 'PROJECT_ID');
            $parentProjectId = DevOpsUtilities::getValueInFilterDataSourceByName($conditions, 'PARENT_PROJECT_ID');
            $data = InstanceProvider::getService()->getProject()->validateParentProjectChoiceByUser($projectId, $parentProjectId);
            $productProject = InstanceProvider::getService()->getProject()->productProjectHasAllowReplicationLts($parentProjectId);
            $response->addDataSet(new DataSet('validateParentProjectChoiceByUser', array($data)));
            $response->addDataSet(new DataSet('productProjectHasAllowReplicationLts', array($productProject)));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function productProjectHasAllowReplicationLts(Request\Filter $request, Response $response)
    {
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $projectId = DevOpsUtilities::getValueInFilterDataSourceByName($conditions, 'PROJECT_ID');
            $productProject = InstanceProvider::getService()->getProject()->productProjectHasAllowReplicationLts($projectId);
            $response->addDataSet(new DataSet('productProjectHasAllowReplicationLts', array($productProject)));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

}
