<?php

namespace Service;


use Doctrine\ORM\EntityManager;
use Util\Utilities;
use Zeedhi\Framework\Cache\Exception;
use Zeedhi\DevOps\Model\Projectmanagement\Entities;

class Allocation {

    protected  $entityManager;
    protected  $environment;
    public  $allocationFactory;

    const ASSIGNER  = Entities\Allocation::TYPE_ASSIGNER;
    const REPORTER  = Entities\Allocation::TYPE_REPORTER;
    const FOLLOWER  = Entities\Allocation::TYPE_FOLLOWER;
    const SPONSOR   = Entities\Allocation::TYPE_SPONSOR;
    const RELEVANT = Entities\Allocation::RELEVANT;
    const NOT_RELEVANT = Entities\Allocation::NOT_RELEVANT;

    public $repositoryAllocation;

    public function __construct(EntityManager $entityManager, \Util\Utilities $environment ,\Factory\Allocation $allocationFactory){
        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->allocationFactory = $allocationFactory;
        $this->repositoryAllocation = $this->entityManager->getRepository('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Allocation');
    }

    public function createAllocation($task, $resource, $type = null,$taskStatus = null, $isNotFlush = false, $isRelevant = self::NOT_RELEVANT){

        if(empty($taskStatus)){
            $taskStatus = Utilities::getTaskStatusNotInitialized();
        }
        if(empty($isRelevant)){
            $isRelevant = self::NOT_RELEVANT;
        }
        $entity = $this->allocationFactory->createAllocation($task, $resource, $type,$taskStatus, $isRelevant);
        $this->entityManager->persist($entity);
        if(!$isNotFlush) {
            $this->entityManager->flush();
        }
        return $entity;

    }
    public function updateAllocation(\Zeedhi\DevOps\Model\Projectmanagement\Entities\Allocation $entity, $task, $resource, $type = null,$taskStatus = null, $isNotFlush = null, $isRelevant = self::NOT_RELEVANT){
        $entity = $this->allocationFactory->updateAllocation($entity, $task, $resource, $type,$taskStatus, $isRelevant);
        $this->entityManager->persist($entity);
        if(!$isNotFlush) {
            $this->entityManager->flush();
        }
        return $entity;

    }

    public function getAllocationByUserAndTask($resourceId, $taskId){
        $allocation = null;
        if(!empty($resourceId) && !empty($taskId)){
            $allocation = $this->entityManager->getRepository('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Allocation')
                        ->findOneBy(array("resource" => $resourceId, "task" => $taskId));
        }
        return $allocation;
    }
    public function setDailyAllocation($task, $date, $hour){
        if($task instanceof \Zeedhi\DevOps\Model\Projectmanagement\Entities\Task){
            $project = $task->getProject();
            if(($project->getProjectType() && $project->getProjectType()->getUsetimeplanning() == 'Y')){
                $allocations = $task->getAllocations();
                /** @var \Zeedhi\DevOps\Model\Projectmanagement\Entities\Allocation $allocation */
                foreach($allocations as $allocation){
                    $dailyAllocation = new \Zeedhi\DevOps\Model\Projectmanagement\Entities\DailyAllocation();
                    $dailyAllocation->setTask($task);
                    $dailyAllocation->setProject($task->getProject());
                    $dailyAllocation->setResource($allocation->getResource());
                    $dailyAllocation->setHourTotalDay($hour);
                    $dailyAllocation->setDay($date);
                    $this->entityManager->persist($dailyAllocation);
                    $this->entityManager->flush($dailyAllocation);
                }
            }
        }
    }

    public function deleteAllDailyAllocation($task){
        if($task instanceof \Zeedhi\DevOps\Model\Projectmanagement\Entities\Task){
            $dailyAllocations = $this->entityManager->getRepository('\Zeedhi\DevOps\Model\Projectmanagement\Entities\DailyAllocation')
                ->findBy(array('task'=>$task));
            foreach($dailyAllocations as $dailyAllocation){
                $this->entityManager->remove($dailyAllocation);
            }
        }
    }
    public function setPlayDateTask($taskId,$resourceId,$initTask){
        /** @var \Zeedhi\DevOps\Model\Projectmanagement\Entities\Allocation $allocation */
        $allocation = $this->repositoryAllocation->findOneBy(array('task'=>$taskId,'resource'=> $resourceId));
        if($allocation && $initTask){
            $allocation->setPlayDate(\DateTime::createFromFormat('d/m/Y H:i:s',$initTask));
        }


    }
    public function updateTaskStatusAllocation($allocation, $taskStatus){
        /** @var \Zeedhi\DevOps\Model\Projectmanagement\Entities\Allocation $allocation */
        $allocation->setTaskStatus($taskStatus);
        $this->entityManager->persist($allocation);
        $this->entityManager->flush();
    }
    public function updateTaskStatusAllocationByTask($task, $taskStatus){
        /** @var \Zeedhi\DevOps\Model\Projectmanagement\Entities\Allocation $allocation */
        $allocations = $this->repositoryAllocation->findBy(array('task'=>$task));
        foreach($allocations as $allocation) {
            $this->updateTaskStatusAllocation($allocation, $taskStatus);
        }
    }

}