<?php
/**
 * Created by PhpStorm.
 * User: tuliomagalhaes
 * Date: 15/02/2016
 * Time: 15:36
 */

namespace Service;

use Util\InstanceProvider;
use Util\Mail\IssueMail;
use Util\Messages;
use Zeedhi\DevOps\Model\Projectmanagement\Entities;
use Zeedhi\DevOps\Model\Projectmanagement\Repositories;
use Doctrine\ORM\EntityManager;
use Util\Utilities;
use Zeedhi\DevOps\Util\Utilities as DevOpsUtilities;

class Comment {
    /**
     * @var EntityManager
     */
    protected $entityManager;
    /**
     * @var \Zeedhi\DevOps\Model\Configurationmanager\Repositories\Subrelease
     */
    protected $subReleaseRepository;
    /**
     * @var \Zeedhi\DevOps\Model\Configurationmanager\Repositories\Release
     */
    protected $releaseRepository;
    /**
     * @var \Zeedhi\DevOps\Model\Configurationmanager\Repositories\Branch
     */
    protected $branchRepository;
    /**
     * @var Repositories\Task
     */
    protected $taskRepository;
    /**
     * @var Repositories\Organization
     */
    protected $organizationRepository;
    /**
     * @var Repositories\TaskBranches
     */
    protected $taskBlanchesRepository;

    /** @var Repositories\Comments  */
    protected $commentsRepository;

    /** @var  Repositories\Allocation $allocationRepository */
    protected $allocationRepository;
    /**
     * @var Utilities
     */
    private $environment;
    /**
     * @var \Factory\Comments
     */
    private $commentsFactory;
    /**
     * @var IssueMail
     */
    private $issueMail;
    /**
     * @var Issue
     */
    private $issueService;
    /**
     * @var Repositories\TaskStatus
     */
    private $taskStatus;
    /**
     * @var Repositories\CommentType
     */
    private $commentTypeRepository;
    /**
     * @var Repositories\TaskProperties
     */
    private $taskPropertiesRepository;

    public $emailsFailedToSend = [];

    public function __construct(EntityManager $entityManager, Utilities $environment,\Factory\Comments $commentsFactory,
                                IssueMail $issueMail, Issue $issueService){
        $this->issueMail = $issueMail;
        $this->environment = $environment;
        $this->issueService = $issueService;
        $this->entityManager = $entityManager;
        $this->commentsFactory = $commentsFactory;
        $this->taskRepository = $entityManager->getRepository(Utilities::ENTITY_DIR.'Task');
        $this->taskStatus = $entityManager->getRepository(Utilities::ENTITY_DIR.'TaskStatus');
        $this->commentsRepository  = $entityManager->getRepository(Utilities::ENTITY_DIR.'Comments');
        $this->commentTypeRepository  = $entityManager->getRepository(Utilities::ENTITY_DIR.'CommentType');
        $this->taskBlanchesRepository = $entityManager->getRepository(Utilities::ENTITY_DIR.'TaskBranches');
        $this->allocationRepository = $this->entityManager->getRepository(Utilities::ENTITY_DIR . 'Allocation');
        $this->branchRepository = $entityManager->getRepository(Utilities::CONFIGURATION_ENTITY_DIR . 'Branch');
        $this->taskPropertiesRepository = $entityManager->getRepository(Utilities::ENTITY_DIR.'TaskProperties');
        $this->releaseRepository = $entityManager->getRepository(Utilities::CONFIGURATION_ENTITY_DIR . 'Release');
        $this->subReleaseRepository = $entityManager->getRepository(Utilities::CONFIGURATION_ENTITY_DIR . 'SubRelease');
        $this->organizationRepository = $entityManager->getRepository(Utilities::ENTITY_DIR . 'Organization');
    }

    public function viewCommentsSaveData($taskId){
		$user = $this->environment->getCurrentUser();
		$issue = $this->taskRepository->find($taskId);
        $lastDate = null;
        if($issue){
            /** @var Entities\Allocation $allocation */
            $allocations = $this->allocationRepository->findBy(array('task' => $issue ,'resource' => $user));
            if(empty($allocations)){
                $allocations = $this->issueService->addFollower($issue, $user);
                $allocations->setLastPostView($this->environment->getCurrentTime());
	            $this->entityManager->persist($allocations);
	            $this->entityManager->flush();
            }else{
            	foreach($allocations as $allocation){
            		$allocation->setLastPostView($this->environment->getCurrentTime());
		            $this->entityManager->persist($allocation);
		            $this->entityManager->flush();
	        		$lastDate = $allocation->getLastPostView();
            	}
            }
        }
	}

    public function setCurrentTaskStatus($taskId){
        /** @var Entities\Task $issue */
		$issue = $this->taskRepository->find($taskId);
        if($issue && $issue->isIssue()){
            $issue->setCurrentTaskStatusIn($this->environment->getCurrentTime());
            $this->entityManager->persist($issue);
            $this->entityManager->flush();
        }
	}

    public function sendCommentsIssuesRelease($releaseId,$version){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        $errorLog = array();

        try {
            $issueDataSet = $this->releaseRepository->getAllIssuesByReleaseToSendMail($releaseId);
            /** @var \Zeedhi\DevOps\Model\Configurationmanager\Entities\Release  $release */
            $release = $this->releaseRepository->find($releaseId);
            $versionString = $release->getVersion().'.'.$release->getRevision().'.'.$release->getBuild();
            /** @var Entities\TaskStatus $homologatedStatus */
            $homologatedStatus = $this->getHomologatedTaskStatus($release->getOrganization()->getId());
            /** @var Entities\TaskProperties $taskProperties */
            $taskProperties = $this->taskPropertiesRepository->findOneBy(array('organization' => $release->getOrganization()->getId()));
            $commentsIssues = array();
            $errorLog[] = 'Issues count: ' . count($issueDataSet);
            foreach ($issueDataSet as $issueRow) {
                /** @var Entities\Task $issue */
                $issue = $this->taskRepository->find($issueRow['ID']);
                if ($issue->getTaskStatus() && $issue->getTaskStatus()->getControl() != Entities\TaskStatus::TASK_STATUS_CONTROL_HOMOLOGATED &&
                    $issue->getTaskStatus()->getId() != $taskProperties->getFinalTaskStatus()->getId()
                ) {
                    $ProjectIdRelated = $this->taskRepository->findReletedProjectIdByIssue($issue->getId(), $issueRow['PROJECTS']);
                    $errorLog[] = 'Related Projects count: ' . count($ProjectIdRelated);
                    if (empty($ProjectIdRelated)) {
                        $errorLog[] = 'Issue: ' . $issue->getId() . ' - "' . $issue->getName() . '" WILL BE HOMOLOGATED';
                        $taskIds = explode(',',$issueRow['TASKS']);
                        $tasks = $this->taskRepository->findBy(array('id' => $taskIds));
                        $versions = $this->taskRepository->findVersionByIssueInRelease($issue->getId(), $releaseId, $version);
                        $commentsIssues = $this->groupTaskCommentByIssue($issue, $tasks, $versions,
                            $commentsIssues);
                    } else {
                        $errorLog[] = 'Issue: ' . $issue->getId() . ' - "' . $issue->getName() . '" WILL NOT BE HOMOLOGATED (PROJECT RESTRICTIONS)';
                    }
                }
            }
            foreach ($commentsIssues as $issueId => $commentsIssue) {
                /** @var Entities\Task $issue */
                $issue = $this->taskRepository->find($issueId);
                $commentType = $this->commentTypeRepository->findOneBy(array('control' => Entities\CommentType::TYPE_AVAILABLE));

                $comment = $this->commentsFactory->createComments($issue,$commentsIssue['comment'],$commentType);
                if($issue->getTaskStatus() != $homologatedStatus){
                    $issue->setCurrentTaskStatusIn((new \DateTime()));
                }
                $issue->setTaskStatus($homologatedStatus);
                $this->entityManager->persist($comment);
                $this->entityManager->persist($issue);
                $this->entityManager->flush();
                $this->viewCommentsSaveData($issueId);

            }
            foreach ($commentsIssues as $issueId => $commentsIssue) {
                $errorLog = $this->sendsMailAdvisingAboutAvailableIssue($issueId,$commentsIssue['emailBody'],
                    $versionString,$errorLog,$commentsIssue['commentVersion'], $commentsIssue['emailBodyDescription']);
            }
            $connection->commit();

            $errorLog = $this->approveContract($commentsIssues, $errorLog);

        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
        return $errorLog;
    }

    public function commentInSpecification(Entities\Task $issue,$specificationComment,$specificationCommentTypeId,
                                           $specificationDocument = null){
        $specificationCommentType  = $this->commentTypeRepository->find($specificationCommentTypeId);
        $comment = $this->commentsFactory->createComments($issue,$specificationComment, $specificationCommentType);
        $issue->setCurrentTaskStatusIn(Utilities::getEnvironment()->getCurrentTime());
        $this->entityManager->persist($issue);
        $this->entityManager->persist($comment);
        $this->entityManager->flush();
        if($specificationDocument){
            Utilities::getAttachmentService()->addDocumentComment($comment->getId(), $specificationDocument);
        }
        $this->sendsMailAdvisingAboutNewIssueComment($issue->getId(), $specificationComment);
    }


    public function sendCommentsIssuesSubRelease($subReleaseId, $version){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        $errorLog = array();
        try {
            $issueDataSet = $this->subReleaseRepository->getAllIssueBySubReleaseToSendMail($subReleaseId);
            /** @var \Zeedhi\DevOps\Model\Configurationmanager\Entities\Subrelease $subRelease */
            $subRelease = $this->subReleaseRepository->find($subReleaseId);
            $versionString = $subRelease->getVersion() . '.' . $subRelease->getRevision() . '.' . $subRelease->getBuild();
            /** @var Entities\TaskStatus $homologatedStatus */
            $homologatedStatus = $this->getHomologatedTaskStatus($subRelease->getOrganization()->getId());
            /**@var Entities\TaskProperties $taskProperties */
            $taskProperties = $this->taskPropertiesRepository->findOneBy(array('organization' => $subRelease->getOrganization()->getId()));
            $isFixRelease = $subRelease->getRelease()->getIsFixRelease() === 'Y' ? true : false;
            $commentsIssues = array();
            $errorLog[] = 'Issues count: ' . count($issueDataSet);
            foreach ($issueDataSet as $issueRow) {
                /** @var Entities\Task $issue */
                $issue = $this->taskRepository->find($issueRow['ID']);
                if ($issue->getTaskStatus() &&
                    $issue->getTaskStatus()->getControl() != Entities\TaskStatus::TASK_STATUS_CONTROL_HOMOLOGATED &&
                    $issue->getTaskStatus()->getId() != $taskProperties->getFinalTaskStatus()->getId()
                ) {
                    $ProjectIdRelated = $this->taskRepository->findReletedProjectIdByIssue($issue->getId(), $issueRow['PROJECTS']);
                    $errorLog[] = 'Related Projects count: ' . count($ProjectIdRelated);
                    if (empty($ProjectIdRelated)) {
                        $errorLog[] = 'Issue: ' . $issue->getId() . ' - "' . $issue->getName() . '" WILL BE HOMOLOGATED';
                        $taskIds = explode(',',$issueRow['TASKS']);
                        $tasks = $this->taskRepository->findBy(array('id' => $taskIds));
                        $versions = $this->taskRepository->findVersionByIssueInSubRelease($issue->getId(), $subReleaseId, $version);
                        $commentsIssues = $this->groupTaskCommentByIssue($issue, $tasks, $versions,
                            $commentsIssues, $isFixRelease);
                    } else {
                        $errorLog[] = 'Issue: ' . $issue->getId() . ' - "' . $issue->getName() . '" WILL NOT BE HOMOLOGATED (PROJECT RESTRICTIONS)';
                    }
                }
            }

            foreach ($commentsIssues as $issueId => $commentsIssue) {
                /** @var Entities\Task $issue */
                $issue = $this->taskRepository->find($issueId);
                if ($issue->getTaskStatus() != $homologatedStatus) {
                    $issue->setCurrentTaskStatusIn((new \DateTime()));
                }
                $issue->setTaskStatus($homologatedStatus);
                $commentType = $this->commentTypeRepository->findOneBy(array('control' => Entities\CommentType::TYPE_AVAILABLE));
                $comment = $this->commentsFactory->createComments($issue, $commentsIssue['comment'], $commentType);
                $this->entityManager->persist($comment);
                $this->entityManager->persist($issue);
                $this->entityManager->flush();
                $this->viewCommentsSaveData($issueId);
            }

            foreach ($commentsIssues as $issueId => $commentsIssue) {
                $errorLog = $this->sendsMailAdvisingAboutAvailableIssue($issueId, $commentsIssue['emailBody'],
                     $versionString, $errorLog, $commentsIssue['commentVersion'], $commentsIssue['emailBodyDescription']);
            }
            $connection->commit();

            $errorLog = $this->approveContract($commentsIssues, $errorLog);

        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }

        return $errorLog;
    }

    public function getControlByCommentTypeId($commentTypeId){
        $control = null;
        /** @var  Entities\CommentType $commentType */
        $commentType = InstanceProvider::getRepository()->getCommentType()->findOneBy(array('id'=>$commentTypeId));
        if($commentType){
            $control = $commentType->getControl();
        }
        return $control;
    }


    /****************************************************************************
     * Sends mail advising to all who commented on a specified issue that there
     * is a new comment available.
     * @param $issueId
     * @param $comment
     */
    public function sendsMailAdvisingAboutNewIssueComment($issueId, $comment){
        $commenterId    = $this->environment->getCurrentUserId();
        $emails         = $this->taskRepository->getRelatedResourcesEmailsByIssueId($issueId, $commenterId);
        $issue          = $this->taskRepository->find($issueId);
        $issueName      = $issue->getName();
        $reporterName   = DevOpsUtilities::getResourceNameForSendingMail($issue, Entities\Allocation::TYPE_REPORTER);
        $sponsor        = DevOpsUtilities::getResourceNameForSendingMail($issue, Entities\Allocation::TYPE_SPONSOR);
        $organizationName = $this->organizationRepository->find($issue->getOrganization())->getName();
        $commenterName  = $this->environment->getCurrentUser()->getName();
        $date           = $this->environment->getCurrentTime()->format(Utilities::FORMAT_DATE_DEFAULT_NO_SECS);
        $issueTitle     = $issueId." - ".$issueName;
        $productName    = $issue->getProject()->getName();
        $mailMessage = "";

        if(!empty($emails)){
            foreach($emails as $email){
                if ($email['EMAIL']) {
                    $task = $this->taskRepository->find($issueId);
                    $mailMessage .= $this->issueMail->buildBodyMessage($task);
                    if(!$this->issueMail->sendCommentMail($email['EMAIL'], $commenterName, $comment, $date, $issueTitle,
                    $issueName, $reporterName, $sponsor, $issueId, $productName, $organizationName)){
                        array_push($this->emailsFailedToSend,$email['EMAIL']);
                    }
                }
            }
        }
    }


    public function sendsMailAdvisingAboutAvailableIssue($issueId, $comment, $version = '.', array &$errorLog = array(), $versions = null, $commentDescription = '', $server = null, $fromDeploy = false){
        $issue = $this->taskRepository->find($issueId);
        $commenterId = $this->environment->getCurrentUserId();
        $emails = $this->taskRepository->getRelatedResourcesEmailsByIssueId($issueId, $commenterId);
        $issueName = $issue->getName();
        $reporter = DevOpsUtilities::getResourceNameForSendingMail($issueId, Entities\Allocation::TYPE_REPORTER);
        $sponsor = DevOpsUtilities::getResourceNameForSendingMail($issueId, Entities\Allocation::TYPE_SPONSOR);
        $organization = $this->organizationRepository->find($issue->getOrganization());
        $organizationName = $organization->getName();
        $isSaas = $this->taskRepository->getIssueReporter($issueId);
        $commenterName = $this->environment->getCurrentUser()->getName();
        $date = $this->environment->getCurrentTime()->format(Utilities::FORMAT_DATE_DEFAULT_NO_SECS);
        $issueTitle = $issueId . " - " . $issueName;
        $productName = $issue->getProject()->getName();
        $fromDevOps = !($fromDeploy == true);

        foreach ($emails as $email) {
            try {
                if(!$fromDevOps || ($fromDevOps && !$isSaas)){
                    $this->issueMail->sendAvailableMail($email['EMAIL'], $issueTitle, $commenterName, $date, $comment,
                    $version, $reporter, $sponsor, $issueName, $issueId, $productName, $versions, $organizationName,
                        false, $commentDescription, $fromDevOps, $fromDeploy, $server,null,  $isSaas);
                }


            } catch (\Exception $e) {
                $email = $email['EMAIL'];
                $errorMessage = $e->getMessage();
                array_push($errorLog," A issue $issueId ($issueName) teve o erro ($errorMessage) ao enviar e-mail para ( $email ) com o assunto $issueTitle e conteudo $comment");
            }
        }
        return $errorLog;
    }
    

    public function setVersionsInComment($versions)
    {
        $commentVersion = '';
        foreach ($versions as $version) {
            $commentVersion .= $version['NAME'] . ' : ' . $version['VERSION'] .'
';
        }
        return $commentVersion;
    }

    public function setVersionsInBodyMail($versions,$issueId, $isFixRelease = false)
    {
        $message = $isFixRelease ? Messages::MAIL_VERSION_FIX : Messages::MAIL_VERSION;
        $commentVersion = $this->issueMail->buildEmailComment(str_replace("%issueId%", $issueId, $message));
        $commentVersion .= '<tr>
    <td class="description">';
        foreach ($versions as $version) {
            $commentVersion .= '
            '.$this->issueMail->buildBodyMessageVersion($version);
        }
        return $commentVersion. '
        <br/></td>
</tr>';
    }


    /**
     * @param Entities\Task $issue
     * @param Entities\Task[] $tasks
     * @param array $versions
     * @param array $comments
     * @return array
     */
    public function groupTaskCommentByIssue(Entities\Task $issue, $tasks,
                                            array $versions, array $comments, $isFixRelease = false){
        if(empty($comments[$issue->getId()])){
            $comment = 'A issue "'.$issue->getName().'" foi liberada no(s) Produto(s) e na(s) versão(ões):
            ' . $this->setVersionsInComment($versions);
            $comments[$issue->getId()]['commentVersion'] = $this->setVersionsInBodyMail($versions, $issue->getId(), $isFixRelease);
            $comments[$issue->getId()]['comment'] = $comment;
            $comments[$issue->getId()]['issueName'] = $issue->getName();
            $comments[$issue->getId()]['versions'] = $versions;
            $comments[$issue->getId()]['emailBodyDescription'] = '';
            $comments[$issue->getId()]['emailBody'] = '';
        }
        $commentString = '
'.$issue->getResponse();
        $comments[$issue->getId()]['comment'] .= $commentString;
        $comments[$issue->getId()]['emailBodyDescription'] .= $this->issueMail->buildBodyMessageIssue($issue->getDescription());
        $comments[$issue->getId()]['emailBody'] .= $this->issueMail->buildBodyMessageIssue($issue->getResponse());
        return $comments;
    }

    /**
     * @param  $organizationId
     * @return null|object
     */
    private function getHomologatedTaskStatus($organizationId)
    {
        return $this->taskStatus->findOneBy(array('control' => Entities\TaskStatus::TASK_STATUS_CONTROL_HOMOLOGATED,
            'usedInIssue' => Entities\TaskStatus::USED_IN_ISSUE_YES,'organization' => $organizationId));
    }

    public function setSingleCommentsAsRead($issues){
        $issuesString = '';
        foreach($issues as $issue){
            if($issuesString){
                $issuesString .= ',' . '\'' . intval($issue) . '\'';
            }else{
                $issuesString = '\'' . intval($issue) . '\'';
            }

        }
        $issueIds = $this->taskRepository->searchForSingleComments($issuesString);
        if($issueIds){
            try{
                foreach($issueIds as $issueId){
                    InstanceProvider::getService()->getTask()->viewCommentsSaveData($issueId["ISSUE_ID"]);
                }
            } catch (\Exception $e) {
                throw $e;
            }
        }

        return $issueIds;
    }

    /**
     * @param array $commentsIssues
     * @param Entities\Task $issue
     * @param array $errorLog
     * @return array
     */
    public function approveContract(array $commentsIssues,array $errorLog)
    {
        foreach ($commentsIssues as $issueId => $commentsIssue) {
            try {
                $organizationId = Utilities::getEnvironment()->getOrganizationId();
                /** @var Entities\Task $issue */
                $issue = $this->taskRepository->find($issueId);
                $isIntegration = Integration\IntegrationSM::integrationSMApproveContract($issueId,$organizationId);
                if($isIntegration){
                    $errorLog[] = 'Issue: ' . $issue->getId() . ' - "' . $issue->getName() . '" THE CONTRACT WAS APPROVED';
                }
            } catch (\Exception $e) {
                $errorMessage = 'Error: ' . $e->getMessage() . ' Error code: ' . $e->getCode();
                $errorLog[] = 'Issue: ' . $issue->getId() . ' - "' . $issue->getName() . '" THE CONTRACT CANNOT BE APPROVED ( ' . $errorMessage . ' )';
            }
        }
        return $errorLog;
    }
}
