<?php
namespace Service\Queue\Task;

use Doctrine\ORM\EntityManager;
use Util\Utilities;
use Zeedhi\DevOps\Model\Projectmanagement\Entities\Project;
use Zeedhi\DevOps\Model\Projectmanagement\Entities\Task;
use Service\Calc\TaskCalculator;
use Service\Calendar\TaskCalendar;
use Zeedhi\DevOps\Model\Projectmanagement\EntitiesInterface\Calc\TaskInterface;
use Zeedhi\DevOps\Model\Projectmanagement\EntitiesInterface\Calendar\TaskCalendarInterface;
use Zeedhi\DevOps\Model\Projectmanagement\EntitiesInterface\Tree\Treeble;
use Zeedhi\Queue\Connection;
use Zeedhi\DevOps\Model\Projectmanagement\Entities;

class Consumer extends \Zeedhi\Queue\Consumer{
    public $dir;
    protected static $queueName = "/queue/pm_calc_date";
    /** @var EntityManager */
    protected $entityManager;
    /** @var TaskCalculator  */
    protected $taskCalculator;
    /** @var  TaskCalendar */
    protected $taskCalendar;
    /** @var String */
    private $DIR_PHP;

    /**
     * @param Connection $connection
     * @param EntityManager $entityManager
     *
     * @param TaskCalculator $taskCalculator
     * @param TaskCalendar $taskCalendar
     * @param $DIR_PHP
     */
    public function __construct(Connection $connection, EntityManager $entityManager, TaskCalculator $taskCalculator,
                                TaskCalendar $taskCalendar,$DIR_PHP) {
        parent::__construct($connection);
        $this->entityManager = $entityManager;
        $this->taskCalculator = $taskCalculator;
        $this->taskCalendar = $taskCalendar;
        $this->DIR_PHP = $DIR_PHP;
    }

    protected function log($message) {
        echo $message, "\n";
    }

    /**
     * @param string $message
     * @return void
     */
    public function consume($message) {
        $DIR_PHP = $this->DIR_PHP;
        $array = json_decode($message, true);
        $className = $array['class'];
        $id = $array['id'];
        $this->log("\nStart task {$className} with id {$id}");
        $console = shell_exec("cd {$this->dir} && $DIR_PHP calcDatesScript.php {$id} {$className}");
        $this->log( $console);
        $this->log("Processed {$className} with id {$id}");
        $this->log('------------------------');
    }

    public function exec($className, $id){
        try{
            $start = microtime(true);
            /** @var \Zeedhi\DevOps\Model\Projectmanagement\EntitiesInterface\Calendar\TaskCalendarInterface $entity */
            $entity = $this->entityManager->getRepository(Utilities::ENTITY_DIR.$className)->findOneBy(array('id' => $id));

            if(!empty($entity)){

                $this->calculateEntity($entity);
                $this->updateDates($entity);
            }
            $end = microtime(true);
            $time = $end-$start;
            $this->log($time);

        }catch (\Exception $e){
            echo $e->getMessage();
        }
    }
    /**
     * @param string $message
     * @return void
     */
    public function consumeCalendar($message) {
        $DIR_PHP = $this->DIR_PHP;
        $array = json_decode($message, true);
        $className = $array['class'];
        $id = $array['id'];
        $this->log("\nStart task {$className} with id {$id}");
        $console = shell_exec("cd {$this->dir}..".DIRECTORY_SEPARATOR."scripts && $DIR_PHP calcDatesScript.php {$id} {$className}");
        $this->log( $console);
        $this->log("Processed {$className} with id {$id}");
        $this->log('------------------------');
    }
    public function setDates($className, $id){
        $start = microtime(true);
        /** @var \Zeedhi\DevOps\Model\Projectmanagement\EntitiesInterface\Calendar\TaskCalendarInterface $entity */
        $entity = $this->entityManager->getRepository('Zeedhi\\DevOps\Model\\Projectmanagement\\Entities\\'.$className)->findOneBy(array('id' => $id));

        if(!empty($entity)){
            $this->updateDates($entity);
        }
        $end = microtime(true);
        $time = $end-$start;
        $this->log($time);
    }

    /**
     * @param $entity
     */
    private function calculateEntity($entity){
        if($entity instanceof Treeble && $entity->allowEndTask() && !$entity->isTimeLine()) {
            $this->taskCalculator->calculate($entity);
            $this->entityManager->persist($entity);
            $entityTreeParent = $entity->getTreeParent();
            if($entityTreeParent){
                $this->taskCalculator->calculate($entityTreeParent);
                $this->entityManager->persist($entityTreeParent);
            }
            $this->entityManager->flush();
        }
        if($entity instanceof Entities\Appropriation && !$entity->getTask()->isTimeLine()){
            $this->taskCalculator->calculate($entity->getTask());
            $this->entityManager->persist($entity->getTask());
            $this->entityManager->flush();
        }
    }

    /**
     * @param TaskCalendarInterface $entity
     */
    private function updateDates($entity){

        $this->entityManager->refresh($entity);

        if ($entity instanceof TaskInterface && $entity->isActive() &&
            (($entity instanceof Project && $entity->getIsIssueProject() !== 'Y') ||
                ($entity instanceof Task && $entity->getIsIssue() === 'N'))) {

            $entity->setInCalc('Y');

            $this->entityManager->persist($entity);
            $this->entityManager->flush($entity);

            /** @var Project $entity */
            if ($entity->isLogicalDeleted()) {
                
                $children = $entity->children();

                if (!$children) {
                    $this->taskCalendar->setDates($entity);
                }
            } else {
                $this->taskCalendar->setDates($entity);
            }
            $entity->setInCalc('N');
            $this->entityManager->persist($entity);
            $this->entityManager->flush();
        }
    }

}